Crate guard_clause

Crate guard_clause 

Macros

guard
Checks that a boolean expression is true at runtime, otherwise performs a return with the second parameter, short-circuiting the operation of the enclosing function.